package de.ozgcloud.eingang.enterprise.entry;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.io.InputStream;

import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.core.io.Resource;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.itvsh.kop.common.test.TestUtils;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.FormHeaderTestFactory;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import de.ozgcloud.eingang.semantik.SemantikAdapter;
import lombok.SneakyThrows;

class EntryControllerTest {

	@Spy
	@InjectMocks
	private EntryController controller;

	@Mock
	private EntryDataMapper mapper;
	@Mock
	private SemantikAdapter semantikAdapter;
	@Mock
	private VorgangNummerSupplier vorgangNummerSupplier;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class ReceiveAntrag {

		private final FormHeader header = FormHeaderTestFactory.createBuilder().vorgangNummer(null).build();
		private final FormData formData = FormDataTestFactory.createBuilder().header(header).build();

		@Captor
		private ArgumentCaptor<InputStream> streamCaptor;
		@Captor
		private ArgumentCaptor<FormData> formDataCaptor;

		private final EntryResponse response = EntryResponseTestFactory.create();

		@BeforeEach
		void init() {
			when(mapper.mapEntryData(any())).thenReturn(formData);
			doReturn(response).when(controller).buildResponse(any(), any());
			when(vorgangNummerSupplier.get()).thenReturn(FormHeaderTestFactory.VORGANG_NUMMER);
		}

		@Test
		@SneakyThrows
		void shouldReturnAccepted() {
			doPostRequest().andExpect(status().isAccepted());
		}

		@Test
		@SneakyThrows
		void shouldCallMapper() {
			var request = TestUtils.loadTextFile("request/simple.json");

			doPostRequest();

			verify(mapper).mapEntryData(streamCaptor.capture());
			var inputBytes = IOUtils.readFully(streamCaptor.getValue(), request.getBytes().length);
			assertThat(inputBytes).hasSameSizeAs(request.getBytes()).isEqualTo(request.getBytes());
		}

		@Test
		void shouldCallSemantikAdapter() {
			doPostRequest();

			verify(semantikAdapter).processFormData(notNull());
		}

		@Test
		void shouldSetVorgangNummer() {
			doPostRequest();

			verify(semantikAdapter).processFormData(formDataCaptor.capture());
			assertThat(formDataCaptor.getValue().getHeader().getVorgangNummer()).isEqualTo(FormHeaderTestFactory.VORGANG_NUMMER);
		}

		@Test
		@SneakyThrows
		void shouldReturnResponse() {
			var response = controller.receiveAntrag(mock(Resource.class));

			assertThat(response).isSameAs(this.response);
		}

		@SneakyThrows
		private ResultActions doPostRequest() {
			return mockMvc.perform(multipart("/antrag")
					.file(new MockMultipartFile("formData", TestUtils.loadTextFile("request/simple.json").getBytes())))
					.andExpect(status().is2xxSuccessful());
		}
	}

	@Nested
	class BuildResponse {

		@Test
		@SneakyThrows
		void shouldCreateResponse() {
			var response = controller.buildResponse(FormDataTestFactory.create(), ResponseVorgangTestFactory.VORGANG_ID);

			assertThat(response).usingRecursiveComparison()
					.ignoringFields("vorgang.statusSince")
					.isEqualTo(EntryResponseTestFactory.create());
		}
	}

}
