/*
 * Copyright (C) 2022-2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import static de.ozgcloud.eingang.common.formdata.FormSolutionsTestFactory.*;
import static de.ozgcloud.eingang.formsolutions.FormSolutionsFilesTestFactory.*;
import static de.ozgcloud.eingang.formsolutions.FormSolutionsRepresentationsMapper.*;
import static org.assertj.core.api.Assertions.*;

import java.io.File;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.itvsh.kop.common.binaryfile.TempFileUtils;
import de.itvsh.kop.common.test.TestUtils;
import de.ozgcloud.eingang.common.formdata.FormSolutionsTestFactory;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import lombok.SneakyThrows;

class FormSolutionsRepresentationsMapperTest {

	@Spy
	private FormSolutionsRepresentationsMapper mapper = new FormSolutionsRepresentationsMapper();

	private File simpleJsonFile;

	@BeforeEach
	void writeJsonFile() {
		simpleJsonFile = TempFileUtils.writeTmpFile(SIMPLE_JSON_DATA);
	}

	@AfterEach
	void delJsonFile() {
		simpleJsonFile.delete();
	}

	@DisplayName("Map representations")
	@Nested
	class TestMapRepresentations {

		@DisplayName("pdf")
		@Nested
		class TestPdfRepresentations {

			@Test
			@SneakyThrows
			void shouldSetContentStrean() {
				var representation = mapRepresentationPdf();

				assertThat(TestUtils.contentStreamToByteArray(representation.getContentStream())).isEqualTo(PDF_DECODED);
			}

			@Test
			void shouldSetHaveContentType() {
				var representation = mapRepresentationPdf();

				assertThat(representation.getContentType()).isEqualTo(PDF_CONTENT_TYPE);
			}

			@Test
			void shouldSetName() {
				var representation = mapRepresentationPdf();

				assertThat(representation.getName()).isEqualTo(FILE_NAME_PDF_REP);
			}

			@Test
			void shouldSetSize() {
				var representation = mapRepresentationPdf();

				assertThat(representation.getSize()).isEqualTo(FormSolutionsFilesTestFactory.PDF_DECODED.length);
			}

			private IncomingFile mapRepresentationPdf() {
				return mapRepresentation().get(0);
			}
		}

		@DisplayName("json")
		@Nested
		class TestJsonRepresentation {

			@Test
			void shouldHaveSize() {
				var map = mapRepresentation();

				assertThat(map).hasSize(2);
			}

			@Test
			@SneakyThrows
			void shouldSetContentStream() {
				var representation = mapRepresentationJson();

				assertThat(TestUtils.contentStreamToByteArray(representation.getContentStream()))
						.isEqualTo(FormSolutionsTestFactory.SIMPLE_JSON_DATA.getBytes());
			}

			@Test
			void shouldSetContentType() {
				var representation = mapRepresentationJson();

				assertThat(representation.getContentType()).isEqualTo(JSON_CONTENT_TYPE);
			}

			@Test
			void shouldSetName() {
				var representation = mapRepresentationJson();

				assertThat(representation.getName()).isEqualTo(FILE_NAME_JSON_REP);
			}

			@Test
			void shouldSetSize() {
				var representation = mapRepresentationJson();

				assertThat(representation.getSize()).isEqualTo(SIMPLE_JSON_DATA.getBytes().length);
			}

			@Test
			void shouldParseJsonOnly() {
				var map = mapper.mapRepresentations(
						FormSolutionsFilesTestFactory.createBuilder().formData(Map.of()).build().getFormData(),
						simpleJsonFile);

				assertThat(map).hasSize(1);
				assertThat(map.get(0).getContentType()).isEqualTo(JSON_CONTENT_TYPE);
			}

			private IncomingFile mapRepresentationJson() {
				return mapRepresentation().get(1);
			}
		}

		private List<IncomingFile> mapRepresentation() {
			return mapper.mapRepresentations(REPRESENTATIONS.getFormData(), simpleJsonFile);
		}
	}

}
