/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.router;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.Collections;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import jakarta.validation.Constraint;
import jakarta.validation.ConstraintValidator;
import jakarta.validation.ConstraintValidatorContext;
import jakarta.validation.Payload;
import jakarta.validation.constraints.NotNull;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.validation.annotation.Validated;

import de.ozgcloud.eingang.router.PlutoListProperties.PlutoListPropertiesConstraint;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
@Configuration
@ConfigurationProperties(prefix = "kop.adapter")
@Validated
@PlutoListPropertiesConstraint
class PlutoListProperties {

	enum FallbackStrategy {
		DENY, FUNDSTELLE
	}

	enum RoutingStrategy {
		MULTI, SINGLE
	}

	@Autowired(required = false)
	private Optional<GrpcClientsProperties> clientProperties = Optional.empty();

	@NotNull
	private RoutingStrategy routingStrategy;
	private Optional<FallbackStrategy> fallbackStrategy = Optional.empty();

	private Optional<String> targetPlutoName = Optional.empty();
	private Optional<String> fundstellePlutoName = Optional.empty();

	private Map<String, String> organisationseinheiten = Collections.emptyMap();

	@Documented
	@Constraint(validatedBy = PlutoListPropertiesValidator.class)
	@Target({ ElementType.TYPE })
	@Retention(RetentionPolicy.RUNTIME)
	static @interface PlutoListPropertiesConstraint {
		String message() default "Routing Configuration invalid";

		Class<?>[] groups() default {};

		Class<? extends Payload>[] payload() default {};
	}

	@AllArgsConstructor
	public static class PlutoListPropertiesValidator implements ConstraintValidator<PlutoListPropertiesConstraint, PlutoListProperties> {

		private static final Predicate<PlutoListProperties> IS_SINGLE_ROUTING = props -> props.getRoutingStrategy() == RoutingStrategy.SINGLE;
		private static final Predicate<PlutoListProperties> HAS_TARGET_NAME = props -> props.getTargetPlutoName().isPresent();

		private static final Predicate<PlutoListProperties> IS_MULTI_ROUTING = props -> props.getRoutingStrategy() == RoutingStrategy.MULTI;
		private static final Predicate<PlutoListProperties> HAS_FALLBACK_STRATEGY = props -> Objects.nonNull(props.getFallbackStrategy());

		private static final Predicate<PlutoListProperties> IS_FALLBACK_TO_FUNDSTELLE = props -> props.getFallbackStrategy()
				.map(strategy -> strategy == FallbackStrategy.FUNDSTELLE).orElse(false);
		private static final Predicate<PlutoListProperties> HAS_FUNDSTELLE = props -> props.getFundstellePlutoName().isPresent();

		@Override
		public boolean isValid(PlutoListProperties value, ConstraintValidatorContext context) {
			return IS_SINGLE_ROUTING.and(HAS_TARGET_NAME)
					.or(IS_MULTI_ROUTING.and(HAS_FALLBACK_STRATEGY))
					.and(IS_FALLBACK_TO_FUNDSTELLE.negate().or(HAS_FUNDSTELLE))
					.and(this::hasAllPlutosConfigured)
					.test(value);
		}

		private boolean hasAllPlutosConfigured(PlutoListProperties props) {
			var clientNames = props.getClientProperties()
					.map(ps -> ps.getClient())
					.map(Map::keySet)
					.orElse(Collections.emptySet());

			return props.getOrganisationseinheiten().values().stream().map(plutoName -> clientNames.contains("pluto-" + plutoName))
					.collect(Collectors.reducing(true, Boolean::logicalAnd));
		}
	}
}
