package de.ozgcloud.eingang.semantik.enginebased.afm;

import static java.util.Objects.*;

import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
public class ZustaendigeStelleMetadataMapper {

	static final String BEHOERDE_METADATA_FILE_NAME = "behoerde_metadata.xml";

	private static final Predicate<IncomingFile> IS_BEHOERDE_METADATA = inFile -> StringUtils.endsWith(inFile.getName(), BEHOERDE_METADATA_FILE_NAME);

	static final String KEY_BEHOERDE_ANZEIGE_NAME = "behoerde_anzeige_name";
	static final String KEY_BEHOERDE_CALLER_ID = "behoerde_caller_id";
	static final String KEY_GEMEINDE_SCHLUESSEL_BP = "gemeinde_schluessel_bp";
	static final String KEY_BEHOERDE_EMAIL = "behoerde_email";
	static final String KEY_AMTLICHER_REGIONALSCHLUESSEL = "amtlicher_regionalschluessel";
	static final String KEY_BEHOERDE_HAUSANSCHRIFT_STRASSE = "behoerde_hausanschrift_strasse";
	static final String KEY_BEHOERDE_HAUSANSCHRIFT_ORT = "behoerde_hausanschrift_ort";
	static final String KEY_BEHOERDE_HAUSANSCHRIFT_PLZ = "behoerde_hausanschrift_plz";
	static final String KEY_BEHOERDE_TELEFON = "behoerde_telefon";

	@Autowired
	private XmlMapperSupplier xmlMapperSupplier;

	public FormData parseZustaendigeStelleData(FormData formData) {
		return formData.getRepresentations().stream().filter(IS_BEHOERDE_METADATA).findAny()
				.map(this::readZustaendigeStelleMetadata)
				.map(this::mapZustaendigeStelle)
				.map(zustaendigeStelle -> formData.toBuilder().zustaendigeStelle(zustaendigeStelle).build())
				.orElse(formData);
	}

	Map<String, String> readZustaendigeStelleMetadata(IncomingFile metadata) {
		return readXmlContent(metadata).map(ZustaendigeStelleData::getFields).map(this::collectToMap).orElse(Collections.emptyMap());
	}

	Optional<ZustaendigeStelleData> readXmlContent(IncomingFile metadata) {
		try {
			return Optional.of(xmlMapperSupplier.getMapper().readValue(metadata.getContentStream(), ZustaendigeStelleData.class));
		} catch (IOException | TechnicalException e) {
			LOG.error("Error parsing {}", BEHOERDE_METADATA_FILE_NAME, e);
		}
		return Optional.empty();
	}

	Map<String, String> collectToMap(List<ZustaendigeStelleData.Field> fields) {
		return fields.stream().filter(field -> nonNull(field.getValue()))
				.collect(Collectors.toMap(ZustaendigeStelleData.Field::getName, ZustaendigeStelleData.Field::getValue));
	}

	ZustaendigeStelle mapZustaendigeStelle(Map<String, String> zustaendigeStelleMetadata) {
		return ZustaendigeStelle.builder()
				.bezeichnung(zustaendigeStelleMetadata.get(KEY_BEHOERDE_ANZEIGE_NAME))
				.organisationseinheitenId(zustaendigeStelleMetadata.get(KEY_BEHOERDE_CALLER_ID))
				.gemeindeSchluessel(zustaendigeStelleMetadata.get(KEY_GEMEINDE_SCHLUESSEL_BP))
				.email(zustaendigeStelleMetadata.get(KEY_BEHOERDE_EMAIL))
				.amtlicherRegionalSchluessel(zustaendigeStelleMetadata.get(KEY_AMTLICHER_REGIONALSCHLUESSEL))
				.hausanschriftStrasse(zustaendigeStelleMetadata.get(KEY_BEHOERDE_HAUSANSCHRIFT_STRASSE))
				.hausanschriftOrt(zustaendigeStelleMetadata.get(KEY_BEHOERDE_HAUSANSCHRIFT_ORT))
				.hausanschriftPlz(zustaendigeStelleMetadata.get(KEY_BEHOERDE_HAUSANSCHRIFT_PLZ))
				.telefon(zustaendigeStelleMetadata.get(KEY_BEHOERDE_TELEFON)).build();
	}

	public boolean isResponsible(FormData formData) {
		return formData.getRepresentations().stream().anyMatch(IS_BEHOERDE_METADATA);
	}
}
