/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import java.util.Collection;
import java.util.List;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.semantik.common.ZipAttachmentReader;
import de.ozgcloud.eingang.semantik.enginebased.FilesMapperHelper;
import lombok.extern.log4j.Log4j2;

@Component
@Log4j2
class FormSolutionsFilesMapper implements FormSolutionsEngineBasedMapper {

	public static final String EXTRAHIERTE_ATTACHMENTS = "Extrahierte Attachments";
	public static final String ZIP_CONTENT_TYPE = "application/zip";

	@Override
	public FormData parseFormData(FormData srcFormData) {
		var formDataBuilder = FilesMapperHelper.getAttachedFileGroups(srcFormData)
				.map(this::collectAttachmentContent)
				.map(fileGroups -> srcFormData.toBuilder().attachments(fileGroups)
						.numberOfAttachments(FilesMapperHelper.countAttachedFiles(fileGroups)))
				.orElseGet(srcFormData::toBuilder);

		FilesMapperHelper.getRepresentations(srcFormData)
				.ifPresent(representations -> formDataBuilder.representations(representations).numberOfRepresentations(representations.size()));
		return FilesMapperHelper.removeProcessedData(formDataBuilder.build());
	}

	protected List<IncomingFileGroup> collectAttachmentContent(List<IncomingFileGroup> attachmentZip) {
		return attachmentZip.stream()
				.flatMap(attachment -> attachment.getFiles().stream())
				.map(file -> ZipAttachmentReader.from(file.getContentStream(), file.getName()))
				.map(this::mapToIncomingFileGroup)
				.toList();
	}

	protected IncomingFileGroup mapToIncomingFileGroup(ZipAttachmentReader zipAttachment) {
		List<IncomingFile> incomingFiles;
		try {
			incomingFiles = zipAttachment.readContent();
			zipAttachment.deleteSourceFile();
		} catch (RuntimeException e) {
			LOG.error("Cannot read source ZIP. Attach it as is.", e);
			incomingFiles = List.of(buildIncomingZipFile(zipAttachment));
		}
		return createIncomingFileGroup(incomingFiles);
	}

	private IncomingFile buildIncomingZipFile(ZipAttachmentReader zipAttachment) {
		return IncomingFile.builder()
				.name(zipAttachment.getSourceFileName())
				.size(zipAttachment.getSourceFileSize())
				.file(zipAttachment.getSourceZip())
				.contentType(ZIP_CONTENT_TYPE)
				.build();
	}

	protected IncomingFileGroup createIncomingFileGroup(Collection<IncomingFile> incomingFiles) {
		return IncomingFileGroup.builder().name(EXTRAHIERTE_ATTACHMENTS).files(incomingFiles).build();
	}

}