package de.ozgcloud.eingang.semantik.enginebased;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataUtils;
import de.ozgcloud.eingang.common.formdata.PostfachAddressTestFactory;
import de.ozgcloud.eingang.common.formdata.ServiceKonto;
import de.ozgcloud.eingang.common.formdata.StringBasedIdentifier;
import de.ozgcloud.eingang.common.formdata.ServiceKonto.PostfachAddress;
import de.ozgcloud.eingang.semantik.enginebased.ServiceKontoBuildHelper;
import de.ozgcloud.eingang.semantik.enginebased.afm.AfmHeaderTestFactory;

class ServiceKontoBuildHelperTest {

	@Spy
	private ServiceKontoBuildHelper helper = new ServiceKontoBuildHelper();

	@DisplayName("service konto")
	@Nested
	class TestServiceKonto {

		private static final FormData FORM_DATA = FormData.builder().formData(AfmHeaderTestFactory.createFormDataMap()).build();

		@DisplayName("with configured postfach")
		@Nested
		class TestWithConfiguredPostfach {

			private static final PostfachAddress POSTFACH_ADDRESS = PostfachAddressTestFactory.create();

			@BeforeEach
			void mockBuildPostfachAddresses() {
				doReturn(List.of(POSTFACH_ADDRESS)).when(helper).buildPostfachAddresses(any(), any());
			}

			@Test
			void shouldContainsType() {
				var serviceKonto = getServiceKonto(FORM_DATA);

				assertThat(serviceKonto.getType()).isEqualTo(ServiceKontoBuildHelper.POSTFACH_TYPE);
			}

			@Test
			void shouldContainsPostfachAddresses() {
				var serviceKonto = getServiceKonto(FORM_DATA);

				assertThat(serviceKonto.getPostfachAddresses()).hasSize(1);
				assertThat(serviceKonto.getPostfachAddresses().get(0)).isEqualTo(POSTFACH_ADDRESS);
			}

			@Test
			void shouldBuildPostfachAddresses() {
				getServiceKonto(FORM_DATA);

				verify(helper).buildPostfachAddresses(any(), any());
			}
		}

		private ServiceKonto getServiceKonto(FormData formData) {
			return helper.buildServiceKonto(AfmHeaderTestFactory.POSTFACH_NAME_ID, formData);
		}

		@DisplayName("postfach addresses")
		@Nested
		class TestBuildPostfachAddresses {

			@DisplayName("with rest_response_name")
			@Nested
			class TestWithRestResponseName {

				@Test
				void shouldCallBuildAddresses() {
					getPostfachAddresses();

					verify(helper).buildOsiPostfachV1Address(any(), anyInt());
				}

				@Test
				void shouldReturnPostfachAddresses() {
					var addresses = getPostfachAddresses();

					assertThat(addresses).hasSize(1);
					assertThat(addresses.get(0).getIdentifier()).isInstanceOf(StringBasedIdentifier.class);
					assertThat(((StringBasedIdentifier) addresses.get(0).getIdentifier()).getPostfachId())
							.isEqualTo(AfmHeaderTestFactory.POSTFACH_NAME_ID);
					assertThat(addresses.get(0).getVersion()).isEqualTo(ServiceKontoBuildHelper.POSTFACH_VERSION);
					assertThat(addresses.get(0).getType()).isEqualTo(PostfachAddressTestFactory.POSTFACH_ADDRESS_TYPE);
				}

				private List<PostfachAddress> getPostfachAddresses() {
					return buildServiceKonto(FORM_DATA).getPostfachAddresses();
				}
			}

			@DisplayName("without rest_response_name")
			@Nested
			class TestWithoutRestResponseName {

				private static final FormData FORM_DATA_WITHOUT_REST_RESPONSE_NAME = FormDataUtils.from(FORM_DATA)
						.remove(ServiceKontoBuildHelper.REST_RESPONSE_NAME).build();

				@Test
				void shouldBuildDefault() {
					getPostfachAddresses();

					verify(helper).buildDefault(AfmHeaderTestFactory.POSTFACH_NAME_ID);
				}

				@Test
				void shouldReturnPostfachAddresses() {
					var addresses = getPostfachAddresses();

					assertThat(addresses).hasSize(1);

					assertThat(addresses.get(0).getIdentifier()).isInstanceOf(StringBasedIdentifier.class);

					assertThat(((StringBasedIdentifier) addresses.get(0).getIdentifier()).getPostfachId())
							.isEqualTo(AfmHeaderTestFactory.POSTFACH_NAME_ID);
					assertThat(addresses.get(0).getVersion()).isEqualTo(ServiceKontoBuildHelper.POSTFACH_VERSION);
					assertThat(addresses.get(0).getType()).isEqualTo(1);
				}

				private List<PostfachAddress> getPostfachAddresses() {
					return buildServiceKonto(FORM_DATA_WITHOUT_REST_RESPONSE_NAME).getPostfachAddresses();
				}
			}

			private ServiceKonto buildServiceKonto(FormData formData) {
				return helper.buildServiceKonto(AfmHeaderTestFactory.POSTFACH_NAME_ID, formData);
			}
		}
	}
}