/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import static de.ozgcloud.eingang.semantik.enginebased.FilesMapperHelper.*;
import static de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsFilesMapper.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;
import java.util.stream.IntStream;

import org.assertj.core.api.InstanceOfAssertFactories;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;
import org.mockito.Mockito;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroupTestFactory;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import de.ozgcloud.eingang.semantik.common.ReadZipException;
import de.ozgcloud.eingang.semantik.common.ZipAttachmentReader;
import de.ozgcloud.eingang.semantik.enginebased.FilesMapperHelper;
import de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsFilesMapper;
import lombok.SneakyThrows;

class FormSolutionsFilesMapperTest {

	private static final String ZIP_CONTENT_TYPE = "application/zip";

	@Spy
	private final FormSolutionsFilesMapper mapper = new FormSolutionsFilesMapper();

	@Nested
	class TestAttachmentMapping {

		private final List<IncomingFileGroup> attachments = List.of(
				IncomingFileGroupTestFactory.createBuilder()
						.name(IncomingFileGroupTestFactory.NAME)
						.clearFiles()
						.files(List.of(IncomingFileTestFactory.createBuilder()
								.name("attachments.zip")
								.contentType(ZIP_CONTENT_TYPE)
								.build()))
						.build());

		private final FormData sourceFormData = FormData.builder()
				.formData(Map.of(FIELD_NAME_MAPPED_FILES, Map.of(ATTACHMENTS, attachments))).build();

		@Test
		void shouldHaveFiles() {
			try (var zipAttachment = Mockito.mockStatic(ZipAttachmentReader.class)) {
				initZipAttachmentMocks(zipAttachment);

				var formData = mapper.parseFormData(sourceFormData);

				assertThat(formData.getAttachments()).hasSize(1).first()
						.extracting(IncomingFileGroup::getFiles, as(InstanceOfAssertFactories.LIST)).hasSize(2);
			}
		}

		@Test
		void shouldSetAttachmentNumber() {
			try (var zipAttachment = Mockito.mockStatic(ZipAttachmentReader.class)) {
				initZipAttachmentMocks(zipAttachment);

				var formData = mapper.parseFormData(sourceFormData);

				assertThat(formData.getNumberOfAttachments()).isEqualTo(2);
			}
		}

		@Test
		void shouldRemoveFilesFromMap() {
			var parsedFormData = mapper.parseFormData(sourceFormData);

			assertThat(parsedFormData.getFormData().get(FilesMapperHelper.FIELD_NAME_MAPPED_FILES)).isNull();
		}
	}

	@Nested
	class TestCollectAttachmentContent {

		@Test
		@DisplayName("should return empty list when there're no attachments")
		void shouldReturnEmptyList() {
			var sourceFileGroups = List.of(IncomingFileGroupTestFactory.createBuilder().clearFiles().build());

			var collectedFileGroups = mapper.collectAttachmentContent(sourceFileGroups);

			assertThat(collectedFileGroups).isEmpty();
		}

		@Test
		void shouldCollectAttachmentContent() {
			mapper.collectAttachmentContent(List.of(IncomingFileGroupTestFactory.create()));

			verify(mapper).mapToIncomingFileGroup(any());
		}
	}

	@SneakyThrows
	private static void initZipAttachmentMocks(MockedStatic<ZipAttachmentReader> zipAttachmentMock) {
		var contentEntries = IntStream.range(0, 2).boxed().map(i -> IncomingFileTestFactory.createBuilder().name("" + i).build()).toList();
		ZipAttachmentReader mock = when(mock(ZipAttachmentReader.class).readContent()).thenReturn(contentEntries).getMock();
		zipAttachmentMock.when(() -> ZipAttachmentReader.from(any(), any())).thenReturn(mock);
	}

	@Nested
	class TestMappingToIncomingFileGroup {

		private ZipAttachmentReader attachment;

		@BeforeEach
		@SneakyThrows
		void setup() {
			attachment = when(mock(ZipAttachmentReader.class).readContent()).thenReturn(List.of(IncomingFileTestFactory.create())).getMock();
		}

		@Test
		void shouldMapAttachment() {
			var incomingFileGroup = mapper.mapToIncomingFileGroup(attachment);

			assertThat(incomingFileGroup.getFiles()).hasSize(1).first().extracting(IncomingFile::getName)
					.isEqualTo(IncomingFileTestFactory.NAME);
		}

		@Test
		@DisplayName("should return source attachment if it's unable to be processed")
		@SneakyThrows
		void shouldReturnSourceAttachment() {
			when(attachment.readContent()).thenThrow(ReadZipException.class);
			when(attachment.getSourceFileName()).thenReturn("source");

			var incomingFileGroup = mapper.mapToIncomingFileGroup(attachment);

			assertThat(incomingFileGroup.getFiles()).hasSize(1).first().extracting(IncomingFile::getName).isEqualTo("source");
		}
	}

	@Nested
	class TestCreateIncomingFileGroup {

		@Test
		@DisplayName("should create incoming file group with default name")
		void shouldCreateFileGroup() {
			var incomingFileGroup = mapper.createIncomingFileGroup(List.of());

			assertThat(incomingFileGroup.getName()).isEqualTo(EXTRAHIERTE_ATTACHMENTS);
		}

		@Test
		@DisplayName("should create new group and add all files")
		void shouldAddAllFiles() {
			var incomingFiles = List.of(IncomingFileTestFactory.create(), IncomingFileTestFactory.create());

			var incomingFileGroup = mapper.createIncomingFileGroup(incomingFiles);

			assertThat(incomingFileGroup.getFiles()).containsExactlyInAnyOrderElementsOf(incomingFiles);
		}
	}

}