package de.ozgcloud.eingang.xta;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

class XtaServiceTest {

	private static final String MESSAGE_TYPE_OTHER = "MESSAGE_TYPE_OTHER";

	@Spy
	@InjectMocks
	private XtaService service;

	@Mock
	private XtaRemoteService remoteService;
	@Mock
	private XtaMessageMapper mapper;

	@Nested
	class TestGetMessagesAsFormData {

		private XtaMessageMetaDatasAndHeader metaData = XtaMessageMetaDatasAndHeaderTestFactory.create();
		private XtaMessageMetaData messageMetaData = XtaMessageMetaDataTestFactory.create();
		private XtaMessage message = XtaMessageTestFactory.create();

		@BeforeEach
		void setup() {
			doReturn(Stream.of(messageMetaData)).when(service).createXtaMessageStream();
		}

		@Test
		void shouldCallCreateStream() {
			service.getMessages();

			verify(service).createXtaMessageStream();
		}

		@Test
		void shouldCallFilterByMessageType() {
			when(remoteService.getMessage(any(XtaMessageId.class))).thenReturn(message);

			service.getMessages().toList();

			verify(service).filterByMessageType(messageMetaData);
		}

		@Test
		void shouldCallGetFormData() {
			when(remoteService.getMessage(any(XtaMessageId.class))).thenReturn(message);
			doReturn(true).when(service).filterByMessageType(messageMetaData);

			service.getMessages().toList();

			verify(service).getFormData(messageMetaData);
		}

		@Test
		void shouldNotCallGetFormData() {
			doReturn(false).when(service).filterByMessageType(messageMetaData);

			service.getMessages().toList();

			verify(service, never()).getFormData(any());
		}
	}

	@Nested
	class TestFilterByMessageType {

		@Test
		void shouldAcceptDFoerdermittel() {
			var metaDataDFoerder = XtaMessageMetaDataTestFactory.create();

			assertThat(service.filterByMessageType(metaDataDFoerder)).isTrue();
		}

		@Test
		void shouldNotAcceptOtherMessageType() {
			var metaDataDFoerder = XtaMessageMetaDataTestFactory.createBuilder().messageType(MESSAGE_TYPE_OTHER).build();

			assertThat(service.filterByMessageType(metaDataDFoerder)).isFalse();
		}
	}

	@Nested
	class TestGetFormData {

		private XtaMessage message = XtaMessageTestFactory.create();

		@Captor
		private ArgumentCaptor<XtaMessage> messageCaptor;

		@BeforeEach
		void init() {
			when(remoteService.getMessage(any(XtaMessageId.class))).thenReturn(message);
		}

		@Test
		void shouldCallRemoteService() {
			service.getFormData(XtaMessageMetaDataTestFactory.create());

			verify(remoteService).getMessage(XtaMessageTestFactory.MESSAGE_ID);
		}

		@Test
		void shouldCallMapper() {
			service.getFormData(XtaMessageMetaDataTestFactory.create());

			verify(mapper).toFormData(any());
		}

		@Test
		void shouldHaveMetaData() {
			XtaMessageMetaData metaData = XtaMessageMetaDataTestFactory.create();
			service.getFormData(metaData);

			verify(mapper).toFormData(messageCaptor.capture());
			assertThat(messageCaptor.getValue().getMetaData()).isNotNull().isEqualTo(metaData);
		}

		@Test
		void shouldReturnMappedResult() {
			var mapped = FormDataTestFactory.create();
			when(mapper.toFormData(any())).thenReturn(mapped);

			var result = service.getFormData(XtaMessageMetaDataTestFactory.create());

			assertThat(result).isSameAs(mapped);
		}
	}

	@Nested
	class TestAcknowledgeReceive {

		@Test
		void shouldCallRemoteService() {
			service.acknowledgeReceive(XtaMessageTestFactory.MESSAGE_ID);

			verify(remoteService).close(XtaMessageTestFactory.MESSAGE_ID);
		}
	}

}
